
:- dynamic(resp/2).

/*---------------------------------------------------------------
   Base de Conhecimento.
----------------------------------------------------------------*/

animal(leopardo):-
	animal(carnivoro),
	tem_cor_amarelo_tostado,
	tem_manchas_escuras.

animal(tigre):-
	animal(carnivoro),
	tem_cor_amarelo_tostado,
	tem_listras_pretas.

animal(girafa):-
	animal(ungulado),
	tem_pernas_longas,
	tem_pescoco_comprido,
	tem_cor_amarelo_tostado,
	tem_manchas_escuras.

animal(zebra):-
	animal(ungulado),
	tem_cor_branca,
	tem_listras_pretas.

animal(avestruz):-
	animal(ave),
	tem_pernas_longas,
	tem_pescoco_comprido,
	tem_cor_preta_branca.

animal(pinguim):-
	animal(ave),
	not(voa),
	nada,
	tem_cor_preta_branca.

animal(albatroz):-
	animal(ave),
	e_um_bom_voador.

animal(carnivoro):-
	animal(mamifero),
	come_carne.

animal(carnivoro):-
	animal(mamifero),
	tem_dentes_pontudos,
	tem_garras,
	tem_olhos_frontais.

animal(mamifero):-
	tem_pelos.

animal(mamifero):-
	da_leite.

animal(ungulado):-
	animal(mamifero),
	tem_casco.

animal(ungulado):-
	animal(mamifero),
	rumina,
	tem_dedos_pares.

animal(ave):-
	tem_penas.

animal(ave):-
	voa,
	bota_ovos.

/*---------------------------------------------------------------
   Relacoes que podem ser perguntadas.
----------------------------------------------------------------*/

perguntavel(tem_pelos).
perguntavel(da_leite).
perguntavel(tem_penas).
perguntavel(voa).
perguntavel(bota_ovos).
perguntavel(come_carne).
perguntavel(tem_dentes_pontudos).
perguntavel(tem_dedos_pares).
perguntavel(tem_olhos_frontais).
perguntavel(tem_casco).
perguntavel(rumina).
perguntavel(tem_listras_pretas).
perguntavel(tem_cor_branca).
perguntavel(tem_cor_preta_branca).
perguntavel(nada).
perguntavel(bom_voador).
perguntavel(tem_pernas_longas).
perguntavel(tem_pescoco_comprido).
perguntavel(tem_cor_amarelo_tostado).
perguntavel(tem_manchas_escuras).


/*---------------------------------------------------------------
   Perguntas que podem ser feitas e o tipo da resposta.
----------------------------------------------------------------*/

pergunta(tem_pelos,'O animal tem pelos?',s-n).
pergunta(da_leite,'O animal da leite?',s-n).
pergunta(tem_penas,'O animal tem penas?',s-n).
pergunta(voa,'O animal voa?',s-n).
pergunta(bota_ovos,'O animal bota ovos?',s-n).
pergunta(come_carne,'O animal come carne?',s-n).
pergunta(tem_dentes_pontudos,'O animal tem dentes pontudos?',s-n).
pergunta(tem_dedos_pares,'O animal tem dedos pares?',s-n).
pergunta(tem_olhos_frontais,'O animal tem olhos frontais?',s-n).
pergunta(tem_casco,'O animal tem casco?',s-n).
pergunta(rumina,'O animal rumina?',s-n).
pergunta(tem_listras_pretas,'O animal tem listas pretas?',s-n).
pergunta(tem_cor_branca,'O animal tem cor branca?',s-n).
pergunta(tem_cor_preta_branca,'O animal tem cor preta e branca?',s-n).
pergunta(nada,'O animal nada?',s-n).
pergunta(bom_voador,'O animal e bom voador?',s-n).
pergunta(tem_pernas_longas,'O animal tem pernas longas?',s-n).
pergunta(tem_pescoco_comprido,'O animal tem pescoco comprido?',s-n).
pergunta(tem_cor_amarelo_tostado,'O animal tem cor amarelo tostado?',s-n).
pergunta(tem_manchas_escuras,'O animal tem manchas escuras?',s-n).


/*---------------------------------------------------------------
   Programa principal.
----------------------------------------------------------------*/

go:-  write('Pense em um animal... e digite ok. .'), nl,
      read(Y),
      prove(animal(X)),nl,
      write('O animal que voce pensou e um(a): '),
      write(X), 
      write(.),
      nl,nl,
      limpa_base,
      !.


/*---------------------------------------------------------------
   Verifica se X eh um predicado pre-definido.
----------------------------------------------------------------*/
sistema(X):- predicate_property(X,built_in).


/*---------------------------------------------------------------
   Verifica se Meta ja foi perguntada.
----------------------------------------------------------------*/
ja_perguntada(Meta):- resp(Meta,Tipo).


/*---------------------------------------------------------------
   Grava Meta na base.
----------------------------------------------------------------*/
lembre_resposta(Meta,Tipo):- assert(resp(Meta,Tipo)).


/*---------------------------------------------------------------
   Faz uma pergunta.
----------------------------------------------------------------*/
faz_pergunta(MetaEnt,Resp):- 
  pergunta(MetaEnt,Perg,TCritica),
  !,
  pergunte(Perg,TCritica,Resp).

pergunte(Perg,TCritica,Resp):-
  write(Perg),
  read(Resp), 
  nl,
  valida_resp(TCritica,Resp),
  !.

pergunte(Perg,TCritica,Resp):- 
  write('Resposta Invalida!!! Tente novamante...'), 
  nl,
  pergunte(Perg,TCritica,Resp).


/*---------------------------------------------------------------
   Verifica se a resposta fornecida eh valida.
----------------------------------------------------------------*/
valida_resp(s-n,sim):- !.
valida_resp(s-n,s):- !.
valida_resp(s-n,nao):- !.
valida_resp(s-n,n):- !.


/*---------------------------------------------------------------
   Verifica se um determinado elemento pertence a uma lista.
----------------------------------------------------------------*/
pertence(X,[X|Y]).
pertence(X,[_|Y]):- pertence(X,Y).


/*---------------------------------------------------------------
   Limpa a base.
----------------------------------------------------------------*/
limpa_base:- retract(resp(_,_)), fail.
limpa_base.


/*---------------------------------------------------------------
   Meta-programa que infere a resposta.
----------------------------------------------------------------*/
prove(true).

prove((MetaA,MetaB)):- 
  prove(MetaA),
  prove(MetaB).

prove((MetaA;MetaB)):- 
  prove(MetaA);
  prove(MetaB).

prove(Meta):- 
  sistema(Meta),
  !,
  Meta.

prove(Meta):-
  clause(Meta,Corpo),
  prove(Corpo), !.

prove(Meta):- 
  perguntavel(Meta),
  not(ja_perguntada(Meta)),
  !,
  faz_pergunta(Meta,Resp),
  lembre_resposta(Meta,Resp),
  (Resp == 'sim'; Resp == 's').
